@@ -3,12 +3,14 @@ use blib;
 use POE;
 use POE::Component::Server::HTTP;
 use Lingua::JA::Categorize;
+use Lingua::JA::Expand;
 use HTML::Feature;
-use Data::Dumper;
 use YAML;
 use Template;
 use CGI;
 use HTTP::Request::AsCGI;
+use FindBin;
+use lib "$FindBin::RealBin/more_train/lib";
 
 my $yaml = YAML::Load( join '', <DATA> );
 
@@ -16,7 +18,7 @@ my $config   = $yaml->{config};
 my $template = $yaml->{template};
 
 my $server = POE::Component::Server::HTTP->new(
-    Port           => 8080,
+    Port           => 7777,
     ContentHandler => {
         '/'           => \&index,
         '/my.js'      => \&js,
@@ -33,8 +35,15 @@ POE::Session->create(
             my $heap = @_[HEAP];
             $heap->{categorizer} = Lingua::JA::Categorize->new(%$config);
             $heap->{categorizer}->load( $config->{state_file} );
-            $heap->{extractor} = HTML::Feature->new;
-            $heap->{tt}        = Template->new;
+            $heap->{extractor} = HTML::Feature->new(
+                engines => [
+                    'MyEngine::Osiete',    # 教えてgoo 専用
+                    'HTML::Feature::Engine::LDRFullFeed',
+                    'HTML::Feature::Engine::GoogleADSection',
+                    'HTML::Feature::Engine::TagStructure',
+                ]
+            );
+            $heap->{tt} = Template->new;
         },
     }
 );
@@ -75,6 +84,7 @@ sub feature {
     my $extractor = $heap->{extractor};
     my $text      = $extractor->parse($url)->text;
     $res->code(RC_OK);
+    $res->headers->header( "Connection" => 'Keep-Alive' );
     $res->content_type('text/html; charset=UTF-8');
     $res->content($text);
 }
@@ -89,8 +99,19 @@ sub categorize {
     my $text        = $q->param("text");
     my $heap        = $poe_kernel->alias_resolve("main")->get_heap();
     my $categorizer = $heap->{categorizer};
-    my $result      = $categorizer->categorize($text);
-
+    my $result;
+    if ( $q->param("expand") ) {
+        my $expander = Lingua::JA::Expand->new;
+        my $word_set = $expander->expand( $text, 20 );
+        my $score    = $categorizer->categorizer->categorize($word_set);
+        $result = Lingua::JA::Categorize::Result->new(
+            word_set => $word_set,
+            score    => $score
+        );
+    }
+    else {
+        $result = $categorizer->categorize($text);
+    }
     my $tt       = $heap->{tt};
     my $template = $template->{score_table};
 
@@ -126,17 +147,28 @@ sub categorize {
         {
             score      => \@score,
             word_set   => \@word_set,
-            categories => \@categories
+            categories => \@categories,
+            expand     => $q->param("expand")
         },
         \my $content
     );
-
-    #print Dumper $tt;
     $res->code(RC_OK);
+    $res->headers->header( "Connection" => 'Keep-Alive' );
     $res->content_type('text/html; charset=UTF-8');
     $res->content($content);
 }
 
+sub expand {
+    my ( $req, $res ) = @_;
+    my $q;
+    {
+        my $cgi = HTTP::Request::AsCGI->new($req)->setup;
+        $q = CGI->new;
+    }
+    my $text = $q->param("text");
+
+}
+
 sub train {
     my ( $req, $res ) = @_;
     my $q;
@@ -147,18 +179,32 @@ sub train {
     my $text        = $q->param("text");
     my $category    = $q->param("category");
     my $num         = $q->param("num");
+    my $expand      = $q->param("expand");
     my $heap        = $poe_kernel->alias_resolve("main")->get_heap();
     my $categorizer = $heap->{categorizer};
     my $brain       = $categorizer->categorizer->brain;
     $num ||= 1;
 
+    my $word_set;
+
+    if ( $q->param("expand") ) {
+        my $expander = Lingua::JA::Expand->new;
+        $word_set = $expander->expand( $text, 20 );
+    }
+    else {
+        $word_set = $categorizer->tokenizer->tokenize( \$text );
+    }
+    $brain->add_instance( attributes => $word_set, label => $category );
     for ( 1 .. $num ) {
-        my $word_set = $categorizer->tokenizer->tokenize( \$text );
-        $brain->add_instance( attributes => $word_set, label => $category );
+        $brain->add_instance(
+            attributes => { dummy => 0 },
+            label      => $category
+        );
     }
     $brain->train;
-    $categorizer->save($config->{state_file});
+    $categorizer->save( $config->{state_file} );
     $res->code(RC_OK);
+    $res->headers->header( "Connection" => 'Keep-Alive' );
     $res->content_type('text/html; charset=UTF-8');
     $res->content("学習成功");
 }
@@ -175,71 +221,93 @@ template:
         <head>
         <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
         <script type="text/javascript" src="http://jqueryjs.googlecode.com/files/jquery-1.3.2.min.js"></script>
-        <script type="text/javascript" src="my.js"></script>
-
+        <script type="text/javascript" src="/my.js"></script>
+        <script type="text/javascript">
+        $(document).ready(function()
+        {
+            //-------------------------------------------------------
+            /*shows the loading div every time we have an Ajax call*/
+            $("#loading").bind("ajaxSend", function(){
+                $(this).show();
+            }).bind("ajaxComplete", function(){
+                $(this).hide();
+            });
+            //-------------------------------------------------------
+        })
+        </script>
         <style type="text/css">
-            .box
-            {
-                margin:10px 0 20px 0;
-            }
-            .rightPos
-            {
-                float:right;
-            }
+        #loading{
+            position: fixed;
+            top: 0;
+            left: 0; /*set it to "right: 0;" to have the bar displaying on the top right corner*/
+            z-index: 5000;
+            background-color: red;
+            font-size: 150%;
+            color: white;
+            padding: 2px;
+        }
+        .box{
+            margin:10px 0 20px 0;
+        }
+        .rightPos{
+            float:right;
+        }
         </style>
 
         <title>Lingua::JA::Categorize</title>
         </head>
         <body>
-
-        <table width=800>
-            <tr>
-                <td><h1>日本語意味分類エンジン</h1></td>
-            </tr>
-        </table>
-
+        <div id="loading" style="display: none;">Loading content...</div>
+        <h1 style="margin-top:40px;">日本語意味分類エンジン</h1>
         <hr />
 
-        <h2>重要部分抽出（ use HTML::Feature ）</h2>
-        <table width=800>
-            <tr>
-                <td>
-                    URL<input type="text" name="url" id="url" size="83">
-                    <button type="submit" onclick="javascript:do_extract()">重要部分抽出</button>
-                    <button type="submit" onclick="javascript:do_clear()">クリア</button>
-                </td>
-            </tr>
-            <tr>
-                <td>
-                    <textarea name="featured" id="featured" cols=90 rows=10></textarea>
-                </td>
-            </tr>
-            <tr>
-                <td>
-                </td>
-            </tr>
-        </table>
+        <div class="box">
+            <table width=800>
+                <tr>
+                    <td>
+                        URL<input type="text" name="url" id="url" size="83">
+                        <button type="submit" onclick="javascript:do_extract()">重要部分抽出</button>
+                        <button type="submit" onclick="javascript:do_clear()">クリア</button>
+        
+                    </td>
+                </tr>
+                <tr>
+                    <td>
+                        <textarea name="featured" id="featured" cols=90 rows=10></textarea>
+                    </td>
+                </tr>
+                <tr>
+        
+                    <td>
+                    </td>
+                </tr>
+            </table>
+        </div>
 
         <table width=800>
             <tr>
                 <td><button type="submit" class="rightPos" onclick="javascript:do_categorize()">分野判定</button></td>
             </tr>
+            <tr>
+                <td><button type="submit" class="rightPos" onclick="javascript:do_expand()">単語拡張</button></td>
+            </tr>
         </table>
-       
+        
         <div class="box" id="score">
         </div>
-
+        
         <div class="box" id="learn">
         </div>
-
+        
         </body>
         </html>
 
+
     score_table: |
         <h2>特徴語（ use Lingua::JA::TFIDF ）</h2>
         <table border=2px width=800>
             [% FOREACH item IN word_set %]
-            <tr><td> [% item.word %] </td><td> [% item.count %] </td></tr>
+            <tr><td><a href="javascript:void(0)" onclick="do_expand('[% item.word %]');"> [% item.word %] </a></td><td> [% item.count %] </td></tr>
             [% END %]
         </table>
         <br><br>
@@ -268,6 +336,7 @@ template:
             <tr>
                 <td>
                     補正回数<input id="train_num" type="text" size="10" value="1">回
+                    <input type="hidden" id="expand" value="[% expand %]">
                 </td>
                 <td>
                     <button class="rightPos" type="submit" onclick="javascript:do_train()">学習</button>
@@ -304,18 +373,45 @@ template:
             });
            
         }
-        function do_train(){
+        function do_expand(keyword){
+            if(!keyword){
+                keyword = $("#featured").val();
+            }
+            var text = encodeURIComponent(keyword);
+
+            if(text.length > 500){
+                alert("単語拡張するには大きすぎます");
+                return;
+            }
+
+            $("#featured").val(keyword);
+            $.ajax({
+                type: "POST",
+                url: "categorize",
+                data: "text=" + text + "&expand=1",
+                success: function(msg){
+                    $("#score").html(msg);
+                }
+            });
+        }
+        function do_train(expand){
             var text     = $("#featured").val();
             var category = $("#train_category").val();
             var num      = $("#train_num").val();
+            var expand   = $("#expand").val();
             $("train").val="";
             $.ajax({
                 type: "POST",
                 url: "train",
-                data: "category=" + category + "&num=" + num + "&text=" + text,
+                data: "category=" + category + "&num=" + num + "&text=" + text + "&expand=" + expand,
                 success: function(msg){
                     $("#train").text("学習成功");
-                    do_categorize();
+                    if(expand){
+                        do_expand(text);
+                    }
+                    else{
+                        do_categorize();
+                    }
                     setTimeout(function(){
                         $("#train_category").val(category);
                     },1000);
